<?php

/**
 * @file
 * Dynamic Resource API sample usage.
 * 
 * This is a sample implementation of dynamic resources that will generate
 * fortunes depending on the path.
 */

// For testing purposes only.
global $fortunes;
$fortunes = array(
  '1.txt' => "A drop of water shall be returned with a burst of spring.",
  '2.txt' => "Fortune does not come twice. Misfortune does not come alone.",
  '3.txt' => "A single day of sub-zero temperature is not enough to create three feet of ice.",
  '4.txt' => "If the wind comes from an empty cave, it's not without a reason.",
  '5.txt' => "Over a long distance, you learn about the strength of your horse; over a long period of time, you get to know what's in a person's heart.",
  '6.txt' => "To hit a dog with a meat-bun.",
  '7.txt' => "You must persevere to accomplish seemingly impossible tasks.",
);

/**
 * Sample of imaterial resource (always being generated).
 * 
 * @package Fortune
 */
class FortuneImaterialResource extends DynamicResource
{
  /**
   * This dummy implementation does need a build.
   * 
   * @see DynamicResourceInterface::build()
   */
  public function build() {}

  /**
   * Pure business stuff.
   * 
   * 
   * @see DynamicResourceInterface::display()
   */
  public function display() {
    echo $this->_text;
  }

  public function getHttpContentType() {
    return "text/plain";
  }

  public function getLifetime() {
    return DynamicResourceInterface::LIFETIME_INFINITE;
  }

  /**
   * Most implementations of imaterial files probably won't need a lock.
   * 
   * @see DynamicResourceInterface::needsLock()
   */
  public function needsLock() {
    return FALSE;
  }

  protected $_text;

  public function setText($text) {
    $this->_text = $text;
  }
}

/**
 * Sample of file resource (stores a static file).
 * 
 * For our usage here, a public file seems to be more than enough.
 * 
 * @package Fortune
 */
class FortuneFileResource extends DynamicResourceFile
{
  protected $_resourcePath;

  public function _buildFileContent() {
    return $this->_text;
  }

  public function getHttpContentType() {
    return "text/plain";
  }

  protected $_text;

  public function setText($text) {
    $this->_text = $text;
  }
} 

/**
 * Sample registry for our fortunes.
 * 
 * @package Fortune
 */
class FortuneResourceProvider extends DynamicResourceProvider
{
  public function getResource($resourcePath, $uri) {
    global $fortunes;

    // Find the file name. This is basically where we would implement a real
    // business logic. For example using CSS or JS aggregation we could fetch
    // a fixed hash that we stored into database in a previous run.
    $parts = explode('/', $resourcePath);
    $name = $parts[1];

    if (2 !== count($parts) || !isset($fortunes[$name])) {
      throw new DynamicResourceNotFoundException("Fortune file '" . $name . "' does not exists.");
    }

    switch ($parts[0]) {
      case 'static':
        $resource = new FortuneFileResource($resourcePath, $uri);
        break;

      case 'volatile':
        $resource = new FortuneImaterialResource($resourcePath, $uri);
        break;

      default:
        throw new DynamicResourceNotFoundException("Fortune file '" . $name . "' does not exists.");
    }

    // We normally have an instance here, set some business options and return it.
    $resource->setText($fortunes[$name]);
    return $resource;
  }
}
